part of 'bolter_provider.dart';

class SyncBuilderController<T> {
  final BolterInterface bolter;
  late T value;
  BolterNotification? _notification;

  SyncBuilderController(this.bolter, this.value);

  void set(T value) {
    bolter.runAndUpdate(exactNotification: _notification, action: () => this.value = value);
  }

  void dispose() {
    _notification = null;
  }
}

class SyncBuilder<T> extends StatefulWidget {
  final T Function()? getter;
  final WidgetBuilder builder;
  final SyncBuilderController? controller;

  const SyncBuilder.getter({
    Key? key,
    required this.getter,
    required this.builder,
  })  : controller = null,
        super(key: key);

  const SyncBuilder.controller({
    Key? key,
    required this.controller,
    required this.builder,
  })  : getter = null,
        super(key: key);

  @override
  _SyncBuilderState<T> createState() => _SyncBuilderState<T>();
}

class _SyncBuilderState<T> extends State<SyncBuilder<T>> {
  late final bolter =
      context.findAncestorWidgetOfExactType<BolterProvider>()?.bolter ?? defaultBolter;

  void _notification() {
    setState(() {});
  }

  @override
  void initState() {
    super.initState();
    widget.controller?._notification = _notification;
    bolter.listen(widget.getter ?? () => widget.controller!.value, _notification);
  }

  @override
  void didUpdateWidget(SyncBuilder<T> oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (oldWidget.getter != widget.getter || oldWidget.controller != widget.controller) {
      bolter.stopListen(_notification);
      bolter.listen(widget.getter ?? () => widget.controller!.value, _notification);
    }
  }

  @override
  Widget build(BuildContext context) {
    return widget.builder(context);
  }

  @override
  void dispose() {
    bolter.stopListen(_notification);
    super.dispose();
  }
}
